/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <memory>
#include <vector>

#include "aidge/utils/Types.h"
#include "aidge/operator/TopK.hpp"
#include "aidge/backend/cpu/data/GetCPUPtr.h"

#include "aidge/backend/cpu/operator/TopKImpl.hpp"
#include "aidge/backend/cpu/operator/TopKImpl_kernels.hpp"

template <>
void Aidge::TopKImpl_cpu::forward() {
    const TopK_Op& op_ = dynamic_cast<const TopK_Op&>(mOp);
    std::int32_t axis = (op_.axis() >= 0) ? op_.axis() : op_.getInput(0)->nbDims() + op_.axis();

    // Find the correct kernel type
    const auto impl = Registrar<TopKImpl_cpu>::create(getBestMatch(getRequiredSpec()));

    // Call kernel
    impl.forward(axis,
                op_.largest(),
                op_.sorted(),
                op_.k(),
                op_.getInput(0)->dims(),
                op_.getInput(0)->getImpl()->rawPtr(),
                op_.getOutput(0)->getImpl()->rawPtr(),
                op_.getOutput(1)->getImpl()->rawPtr());
}

template <>
void Aidge::TopKImpl_cpu::backward() {
    AIDGE_THROW_OR_ABORT(std::runtime_error, "Backward not yet implemented for TopK_Op on backend cpu");
}
